package ex2;

import java.io.IOException;
import javax.microedition.media.MediaException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.media.Manager;
import javax.microedition.media.Player;
import javax.microedition.media.control.VideoControl;

/**
 * This midlet demonstrates how camera streams can be displayed to
 * the screen and how single pictures can be captured. This creates
 * the effect of a photo camera.
 *
 * @author  Alexander De Luca, Gregor Broll
 */
public class CameraMidlet extends MIDlet implements CommandListener {
    private static final Command CLOSE_COMMAND = new Command("close", Command.EXIT, 1);
    private static final Command PHOTO_COMMAND = new Command("shoot",Command.SCREEN, 1);
    private static final Command BACK_COMMAND = new Command("back",Command.SCREEN, 1);
    private Form cameraForm;
    private Display display;
    private Player player;
    private VideoControl videoControl;
    
    private void createForm() {
        display = Display.getDisplay(this);
        cameraForm = new Form("Camera");
        cameraForm.addCommand(CLOSE_COMMAND);
        cameraForm.addCommand(PHOTO_COMMAND);
        cameraForm.setCommandListener(this);
        this.initPlayer();
        display.setCurrent(cameraForm);
    }
    
    
    /**
     * Initializes the player and the videocontrol.
     */
    private void initPlayer() {
        try {
            player = Manager.createPlayer("capture://video");
            player.realize();
            videoControl = (VideoControl)player.getControl("VideoControl");
            if(videoControl!=null) {
                 Item videoItem = (Item)(videoControl.initDisplayMode(VideoControl.USE_GUI_PRIMITIVE,null));
                 videoItem.setLayout(Item.LAYOUT_CENTER | Item.LAYOUT_NEWLINE_AFTER);
                 cameraForm.append(videoItem);
            }
            player.start();
        } catch (MediaException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
    
    /**
     * Creates a snapshot.
     */
    private void makeSnapshot() {
        new Thread() { // anonymous class definition just for ease of usage
            public void run() {
                try {
                    // synchronize due to multiple button pushing
                    synchronized(CameraMidlet.this) {
                        byte[] imageData;
                        imageData = videoControl.getSnapshot(null);
                        CameraMidlet.this.showPicture(imageData);
                    }
                } catch(MediaException e) {
                    System.out.println(e.toString());
                }
            }
        }.start();
    }
    
    /**
     * Displays a picture to the screen.
     *
     * @param imageData The recorded byte array that includes the picture data.
     */
    private void showPicture(byte[] imageData) {
        Form photoForm = new Form("Snapshot");
        Image photo = Image.createImage(imageData, 0, imageData.length);
        // create an Item for more efficient centering
        ImageItem photoItem = new ImageItem(null,photo,ImageItem.LAYOUT_CENTER,null);
        photoForm.append(photoItem);
        photoForm.addCommand(BACK_COMMAND);
        photoForm.setCommandListener(this);
        display.setCurrent(photoForm);
    }
    
    public void commandAction(Command comm, Displayable disp) {
        if(comm == CLOSE_COMMAND)
            this.exitMIDlet();
        else if(comm == PHOTO_COMMAND) {
            this.makeSnapshot();
        } else if(comm == BACK_COMMAND) {
            display.setCurrent(cameraForm);
        }
    }
    
    /**
     * Free resources and close the midlet.
     */
    public void exitMIDlet() {
        player.close();
        display.setCurrent(null);
        destroyApp(true);
        notifyDestroyed();
    }
    
    public void startApp() {
        this.createForm();
    }
    
    public void pauseApp() {
    }
    
    public void destroyApp(boolean unconditional) {
    }
}
